###########################################################################
#  DSpace: Product to embed a DSpace repository within a Plone site.
#
#  https://svn.plone.org/svn/collective/DSpace
#
# This is a product that enables the user to connect to
# a default DSpace application. This application is a digital repository
# for articles, books, courseware, journals, websites and thesis material.
#
#  Copyright (C) 2007  Cranfield University, UK
#  Written by: The Development Team, DCMT Cranfield University
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
###########################################################################

from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *
from Products.Archetypes.debug import log
from Products.ATContentTypes.content.base import ATCTOrderedFolder
from Products.ATContentTypes.content.schemata import ATContentTypeSchema
from Products.ATContentTypes.content.schemata import finalizeATCTSchema
from Products.ATContentTypes.lib.constraintypes import ConstrainTypesMixinSchema
from Products.CMFCore.DirectoryView import createDirectoryView
from Acquisition import aq_base, aq_inner, aq_parent

from Products.DSpace import permissions

HeaderSchema = ATContentTypeSchema.copy()
HeaderSchema = HeaderSchema + ConstrainTypesMixinSchema
schema = HeaderSchema + Schema((

    StringField('dbName',
        required=1,
        searchable=0,
        widget=StringWidget(
            description="Enter the name of the database you are connecting too.",
            label="Database Name",
        ),
    ),

    StringField('dbUser',
        required=1,
        searchable=0,
        widget=StringWidget(
            description="Enter the name of the user connecting to this database session",
            label="User",
        ),
    ),

    StringField('dbHost',
        required=0,
        searchable=0,
        widget=StringWidget(
            description="Enter the database host details",
            label="Host",
        ),
    ),

    StringField('dbPort',
        default='8080',
        searchable=0,
        widget=StringWidget(
            description="Enter the port number used for the DSpace Tomcat configuration (if applicable).",
            label="Port Number",
        ),
    ),

    StringField('dbDirectory',
        searchable=0,
        widget=StringWidget(
            description="Enter specific directory structure after host and port configuration (if applicable).",
            label="Directory",
        ),
    ),

    StringField('dbPassword',
        required=0,
        searchable=0,
        widget=PasswordWidget(
            description="Enter the password for this connection (if applicable)",
            label="Password",
        )
    ),

    StringField('dbPasswordConfirm',
        required=0,
        searchable=0,
        widget=PasswordWidget(
            description="Enter the password again for confirmation",
            label="Confirm Password",
        )
    ),

))

finalizeATCTSchema(schema, folderish=True, moveDiscussion=False)

##Further Schema customisations
schema['title'].widget.description = """Enter a title for the DSpace instance."""
schema['title'].widget.label = "DSpace instance title"
schema["description"].widget.description = """Enter a brief description of the dspace instance."""
schema["description"].widget.label = """DSpace instance name"""


class DSpaceInstance(ATCTOrderedFolder):
    """A DSpace instance"""
    schema = schema
    #actions = ATCTOrderedFolder.actions
    #archetypes_name = portal_type = "DSpace Instance"
    _at_rename_after_creation = True

    security = ClassSecurityInfo()

    security.declarePrivate('post_validate')
    def post_validate(self, REQUEST=None, errors=None):
        """Validates passwords"""
        form = REQUEST.form
        if form.has_key("dbPassword") or form.has_key("dbPasswordConfirm"):
            initialdbpassword = REQUEST.get('dbPassword', None)
            confirmdbpassword = REQUEST.get('dbPasswordConfirm', None)

            if not initialdbpassword and not confirmdbpassword:
                errors['dbPassword'] = "An inital password must be set"

            if initialdbpassword or confirmdbpassword:
                if str(initialdbpassword) != str(confirmdbpassword):
                    errors['dbPassword'] = "Passwords are not the same, please re-enter"
                    errors['dbPasswordConfirm'] = "Passwords are not the same, please re-enter"


    security.declarePrivate('at_post_create_script')
    def at_post_create_script(self):
        self.updateDbaseValues()
        self.updateSkins()
        self.updateLeftSlots()

    security.declarePrivate('at_post_edit_script')
    def at_post_edit_script(self):
        self.updateDbaseValues()


    security.declarePrivate('updateDbaseValues')
    def updateDbaseValues(self, **kwargs):
        self.updateZsycopgConnection(thisdb=self.getDbName(), thisdbuser=self.getDbUser(),
                                     thisdbhost=self.getDbHost(),
                                     thisdbpassword=self.getDbPassword())

    security.declarePrivate('updateSkins')
    def updateSkins(self):
        if 'system' not in self.objectIds():
            createDirectoryView(self, 'DSpace/skins/DSpace', 'system')

    security.declarePrivate('updateLeftSlots')
    def updateLeftSlots(self):
        from Products.CMFPlone.migrations.migration_util import safeEditProperty
        left_slots = ['here/system/portlet_browse/macros/portlet',]
        safeEditProperty(self, 'left_slots', left_slots, 'lines')

    security.declarePrivate('updateZsycopgConnection')
    def updateZsycopgConnection(self, thisdb=None, thisdbuser=None, thisdbhost=None, thistitle=None,
                                thisdbpassword=None):

        """ The following 'if' statements are used with ditto characters otherwise there is a problem with the
        connection string """

        if thisdbhost == '':
            thisdbhost = thisdbhost + "''"

        if thisdbpassword == '':
            thisdbpassword = thisdbpassword + "''"

        required_string = 'dbname=' + thisdb + ' user=' + thisdbuser + ' host='+ thisdbhost + ' password=' + thisdbpassword

        if 'DSpace' not in self.objectIds():
            from Products.ZPsycopgDA.DA import manage_addZPsycopgConnection
            manage_addZPsycopgConnection(self,
                                 id='DSpace',
                                 title='testme',
                                 connection_string=required_string,)
        else:
            DSpace_object = self['DSpace']
            DSpace_object.manage_edit('Edit-test', connection_string=required_string)

    security.declarePublic('canSetDefaultPage')
    def canSetDefaultPage(self):
        """
        Override BrowserDefaultMixin.
        """
        return False

    security.declarePublic('canSetConstrainTypes')
    def canSetConstrainTypes(self):
        """
        Override ConstrainTypesMixin.
        """
        return False

    def __before_publishing_traverse__(self, other, REQUEST=None):
        """
        pass request to contained system folder, unless it is a method for this object
        """
        if REQUEST is None:
            REQUEST = other
        stack = REQUEST.get('TraversalRequestNameStack', [])
        if stack:
            if hasattr(self.aq_inner.aq_explicit, stack[0]):
                # if in the folder, return that
                return
            elif not hasattr(self, 'system'):
                # no system folder
                return
            elif hasattr(self.system.aq_inner.aq_explicit, stack[0]):
                if stack[0][:6] == 'manage':
                    #return the right thing in ZMI
                    return
                else:
                    stack.append('system')
                    REQUEST.set('TraversalRequestNameStack', stack)

registerType(DSpaceInstance)
